"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
var redux_1 = require("redux");
var ModuleManager_1 = require("./Managers/ModuleManager");
var RefCountedManager_1 = require("./Managers/RefCountedManager");
var MiddlewareManager_1 = require("./Managers/MiddlewareManager");
/**
 * Adds dynamic module management capabilities to a redux store.
 * @param extensions: Optional. Any extensions for the store e.g. to support redux-saga or redux-thunk
 * @param initialModules: Optional. Any modules to bootstrap the store with
 */
function moduleEnhancer(extensions, initialModules) {
    if (extensions === void 0) { extensions = []; }
    if (initialModules === void 0) { initialModules = []; }
    return function (createStore) {
        return function (baseReducer, preloadedState, baseEnhancer) {
            // get middlewares from extensions if any
            var extensionMiddlewares = extensions.reduce(function (mw, p) {
                if (p.middleware) {
                    mw.push.apply(mw, p.middleware);
                }
                return mw;
            }, []);
            // create manager to  manage dynamic middlewares
            var middlewareManager = RefCountedManager_1.getRefCountedManager(MiddlewareManager_1.getMiddlewareManager(), function (a, b) { return a === b; });
            // create module manager
            var moduleManager = RefCountedManager_1.getRefCountedManager(ModuleManager_1.getModuleManager(middlewareManager, extensions), function (a, b) { return a.id === b.id; });
            // Create module enhancer to manage extensions and dynamic middlewares
            var moduleEnhancer = redux_1.compose(redux_1.applyMiddleware.apply(void 0, extensionMiddlewares.concat([middlewareManager.enhancer])));
            // compose the moduleEnahancer with the enhancers passed, the order matters as we want to be additive
            // so the right parameter is the enhancer we received
            var composedEnhancer = (baseEnhancer ?
                redux_1.compose(moduleEnhancer, baseEnhancer) :
                moduleEnhancer);
            // build a chained reducer
            var chainedReducer = function (state, action) {
                // call the passed in reducer first
                var intermediateState = baseReducer ? baseReducer(state, action) : state;
                // then delegate to the module managers reducer
                return moduleManager.getReducer(intermediateState, action);
            };
            // Create the store
            var store = createStore(chainedReducer, preloadedState, composedEnhancer);
            // module manager will use the dispatch from store to dispatch initial and final actions
            moduleManager.setDispatch(store.dispatch);
            // adds given modules to mouldManager
            var addModules = function (modulesToBeAdded) {
                moduleManager.add(modulesToBeAdded);
                return {
                    remove: function () {
                        moduleManager.remove(modulesToBeAdded);
                    }
                };
            };
            // Adds the module to the module manager
            var addModule = function (moduleToBeAdded) {
                return addModules([moduleToBeAdded]);
            };
            var dispose = function () {
                // get all added modules and remove them
                moduleManager.dispose();
                middlewareManager.dispose();
                extensions.forEach(function (p) {
                    if (p.dispose) {
                        p.dispose();
                    }
                });
            };
            // Allow extensions to react when modules are added
            extensions.forEach(function (p) {
                if (p.onModuleManagerCreated) {
                    p.onModuleManagerCreated({
                        addModule: addModule,
                        addModules: addModules
                    });
                }
            });
            var moduleStore = {
                addModule: addModule,
                addModules: addModules,
                dispose: dispose
            };
            // Add the initial modules
            moduleStore.addModules(initialModules);
            return __assign({}, store, moduleStore);
        };
    };
}
exports.moduleEnhancer = moduleEnhancer;
